`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 2020/04/22 22:03:55
// Design Name: 
// Module Name: fsm
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////


`define STOP  2'b00 // stop state
`define START 2'b01 // start state
`define PAUSE 2'b10 // pause state
`define RESET 2'b11 // reset  state
`define ENABLED  1'b1
`define DISABLED 1'b0

module fsm(
    input clk,
    input pb, // start/stop push bottom input
    input pb_pause,   // pause/resume push bottom input
    input DIP_set, // set mode DIP switch input
    output reg rst_n, // reset
    output reg cnt_en // count enable
    );
    reg [1:0]state; // current state
    reg [1:0]next_state; // next state

// FSM state decision
always@*
    case (state)
        `STOP:
            if (pb == 1)
                begin
                    next_state = `RESET;
                    cnt_en = `DISABLED;
                    rst_n = 1;
                end
            else if (pb_pause == 1)
                begin
                    next_state = `START;
                    cnt_en = `ENABLED;
                    rst_n = 1;
                end
            else if (DIP_set == 1)
                begin
                    next_state = `RESET;
                    cnt_en = `DISABLED;
                    rst_n = 1;
                end
            else 
                begin
                   next_state = `STOP;
                   cnt_en = `DISABLED;
                   rst_n = 1;
                end
        `START:
            if (pb == 1)
                begin
                    next_state = `STOP;
                    cnt_en = `DISABLED;
                    rst_n = 1;
                end
            else if (pb_pause == 1)
                begin
                    next_state = `STOP;
                    cnt_en = `DISABLED;
                    rst_n = 1;
                end
            else if (DIP_set == 1)
                begin
                    next_state = `RESET;
                    cnt_en = `DISABLED;
                    rst_n = 1;
                end
            else
                begin
                    next_state = `START;
                    cnt_en = `ENABLED;
                    rst_n = 1;
                end
        `RESET:
            if (DIP_set == 1)
                begin
                    next_state = `RESET;
                    cnt_en = `DISABLED;
                    rst_n = 1;
                end
            else 
                begin
                    next_state = `START;
                    cnt_en = `ENABLED;
                    rst_n = 0;
                end
        default:
            begin
                next_state = `RESET;
                cnt_en = `DISABLED;
                rst_n = 0;
            end
    endcase

// FSM state transition
always@(posedge clk or negedge rst_n)
    if (~rst_n)
        state <= `START;
    else
        state <= next_state;

endmodule
