`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 2020/04/22 22:31:11
// Design Name: 
// Module Name: top
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////


module top(
    input clk,
    input pb,
    input DIP_set,
    input pb_pause,
    input set_sec,
    input set_min,
    output [7:0]segs,
    output [3:0]ssd_ctl,
    output [15:0]led
    );
    wire clk_1Hz, clk_100Hz, clk_10Hz;
    wire [1:0]ssd_ctl_en;
    wire pb_d, pause_d, sec_d, min_d;
    wire pb_pulse, pause_pulse, sec_pulse, min_pulse;
    wire carry_sec, carry_min;
    wire cnt_en;
    wire rst_n;
    wire set_en;
    wire rst_set;
    wire [3:0]cnt3, cnt2, cnt1, cnt0;
    wire [3:0]value3, value2, value1, value0;
    wire [3:0]digit3, digit2, digit1, digit0;
    wire borrow0, borrow1, borrow2;
    wire [3:0]ssd_in;
    wire is_over;
    wire borrow_digit3 = borrow0 & borrow1 & borrow2;
    wire borrow_digit2 = borrow0 & borrow1;

    freq_div U0(
        .clk(clk),
        .rst_n(rst_n),
        .clk_ctl(ssd_ctl_en),
        .clk_1Hz(clk_1Hz),
        .clk_100Hz(clk_100Hz),
        .clk_10Hz(clk_10Hz)
    );

    debounce U_pb(
        .pb_in(pb),
        .clk(clk_100Hz),
        .rst_n(rst_n),
        .pb_debounced(pb_d)
    );
    
    debounce U_pause(
        .pb_in(pb_pause),
        .clk(clk_100Hz),
        .rst_n(rst_n),
        .pb_debounced(pause_d)
    );

    debounce U_sec(
        .pb_in(set_sec),
        .clk(clk_100Hz),
        .rst_n(rst_n),
        .pb_debounced(sec_d)
    );

    debounce U_min(
        .pb_in(set_min),
        .clk(clk_100Hz),
        .rst_n(rst_n),
        .pb_debounced(min_d)
    );

    one_pulse U_pb_d(
        .in_trig(pb_d),
        .clk(clk_10Hz),
        .rst_n(rst_n),
        .one_pulse(pb_pulse)
    );

    one_pulse U_pause_d(
        .in_trig(pause_d),
        .clk(clk_10Hz),
        .rst_n(rst_n),
        .one_pulse(pause_pulse)
    );

    one_pulse U_sec_d(
        .in_trig(sec_d),
        .clk(clk_100Hz),
        .rst_n(rst_n),
        .one_pulse(sec_pulse)
    );

    one_pulse U_min_d(
        .in_trig(min_d),
        .clk(clk_100Hz),
        .rst_n(rst_n),
        .one_pulse(min_pulse)
    );

    fsm U1(
        .clk(clk_10Hz),
        .pb(pb_pulse),
        .pb_pause(pause_pulse),
        .DIP_set(DIP_set),
        .cnt_en(cnt_en),
        .rst_n(rst_n)
    );

    fsm_setting U2(
        .clk(clk_100Hz),
        .rst_n(rst_n),
        .DIP_set(DIP_set),
        .set_en(set_en),
        .rst_set(rst_set)
    );

    setting U_d0(
        .clk(clk_100Hz),
        .rst_n(rst_set),
        .increase(sec_pulse),
        .set_en(set_en),
        .limit(4'd9),
        .value(value0),
        .carry(carry_sec)
    );

    setting U_d1(
        .clk(clk_100Hz),
        .rst_n(rst_set),
        .increase(carry_sec),
        .set_en(set_en),
        .limit(4'd5),
        .value(value1)
    );

    setting_d2 U_d2(
        .clk(clk_100Hz),
        .rst_n(rst_set),
        .increase(min_pulse),
        .set_en(set_en),
        .limit(4'd9),
        .value_d3(value3),
        .value_d2(value2),
        .carry(carry_min)
    );

    setting_d3 U_d3(
        .clk(clk_100Hz),
        .rst_n(rst_set),
        .increase(carry_min),
        .set_en(set_en),
        .limit(4'd2),
        .value_d3(value3),
        .value_d2(value2),
        .min_pulse(min_pulse)
    );

    BCD_down_cnt U_digit0(
        .clk(clk_1Hz),
        .rst_n(rst_n),
        .decrease(cnt_en),
        .limit(4'd9),
        .setting(value0),
        .is_over(is_over),
        .value(cnt0),
        .borrow(borrow0)
    );

    BCD_down_cnt U_digit1(
        .clk(clk_1Hz),
        .rst_n(rst_n),
        .decrease(borrow0),
        .limit(4'd5),
        .setting(value1),
        .is_over(is_over),
        .value(cnt1),
        .borrow(borrow1)
    );

    BCD_down_cnt U_digit2(
        .clk(clk_1Hz),
        .rst_n(rst_n),
        .decrease(borrow_digit2),
        .limit(4'd9),
        .setting(value2),
        .is_over(is_over),
        .value(cnt2),
        .borrow(borrow2)
    );

    BCD_down_cnt U_digit3(
        .clk(clk_1Hz),
        .rst_n(rst_n),
        .decrease(borrow_digit3),
        .limit(4'd2),
        .setting(value3),
        .is_over(is_over),
        .value(cnt3)
    );

    MUX U6(
        .digit3(digit3),
        .digit2(digit2),
        .digit1(digit1),
        .digit0(digit0),
        .mode(set_en),
        .digit3_cnt(cnt3),
        .digit2_cnt(cnt2),
        .digit1_cnt(cnt1),
        .digit0_cnt(cnt0),
        .digit3_set(value3),
        .digit2_set(value2),
        .digit1_set(value1),
        .digit0_set(value0)
    );

    scan_ctl U7(
        .ssd_ctl_en(ssd_ctl_en),
        .in3(digit3),
        .in2(digit2),
        .in1(digit1),
        .in0(digit0),
        .ssd_ctl(ssd_ctl),
        .ssd_in(ssd_in)
    );
    
    display U8(
        .in(ssd_in),
        .segs(segs)
    );

    is_over U9(
        .in3(cnt3),
        .in2(cnt2),
        .in1(cnt1),
        .in0(cnt0),
        .is_over(is_over)
    );

    LED U10(
        .is_over(is_over),
        .led(led)
    );

endmodule
